
;+

function hsi_pixon_gof_calc,data,recon_data,residuals,snr_sigma2,r2s, $
                            poisson=poisson,iobj=iobj,dobj=dobj, $
                            a2d=a2d,harmonics=harmonics,setunits=setunits

;NAME:
;     HSI_PIXON_GOF_CALC
;PURPOSE:
;     Calculates the goodness of fit parameter for the Pixon reconstruction.
;     Uses either Poisson or Gaussian statistics.
;CATEGORY:
;CALLING SEQUENCE:
;     gof = hsi_pixon_gof_calc(data,recon_data,residuals,snr_sigma2,r2s,/poisson)
;INPUTS:
;     data = data vector
;     recon_data = modulation profile for the current image
;     residuals = mprofile-data (only used if poisson is NOT set)
;     snr_sigma2 = noise (only used if poisson is NOT set)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;OUTPUTS:
;     gof = goodness of fit parameter
;     r2s = vector for use in the gradient calculations, only computed if the
;           parameter is present, so if you don't need it, don't include it
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;MODIFICATION HISTORY:
;     T. Metcalf 1999-Dec-15
;     T. Metcalf 2002-Jan-03 Added a check for negative recon_data when
;                            /poisson is set.
;-


   if keyword_set(poisson) then begin

      ; There seem to be some cases where the recon data can be slightly
      ; negative.  This should not be the case, but check for it here.
      good = where(recon_data GT 0.0 AND data GT 0.0)
      data_good = data[good]
      recon_data_good = recon_data[good]

      ; What I derived for HXT, which now appears to be wrong ...
      ;;if n_params() GE 5 then r2s = alog(recon_data/data)
      ;;gof = total(data - recon_data + recon_data*alog(recon_data/data))

      ; What Ed Schmahl derived from the Cash paper (ApJ, 228, 939, 1979)
      if n_params() GE 5 then begin
         r2s = 2.0*(1.-data/recon_data)
      endif
      gof = -2.0*total(data_good - recon_data_good + $
                       data_good*alog(recon_data_good/data_good))
   endif else begin 
      if n_params() GE 5 then r2s = 2.0*residuals*snr_sigma2
      gof = total(snr_sigma2*(residuals^2))
   endelse

   return,gof

end
